/**********************************************************************
 *
 * fileutil.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *********************************************************************/

#include "fileutil.h"

#if defined(GSDL_USE_OBJECTSPACE)
#  include <ospace\std\iostream>
#  include <ospace\std\fstream>
#elif defined(GSDL_USE_IOS_H)
#  include <iostream.h>
#  include <fstream.h>
#else
#  include <iostream>
#  include <fstream>
#endif


// returns the proper concatenation of the two paths
text_t filename_cat (text_t path1, text_t path2) {
  text_t::iterator here;
  text_t::iterator begin;
  text_t::iterator end;

  // make sure there is just one slash, of the correct type,
  // at the end of path1 (unless path1 is an empty string).
  if (!path1.empty()) {
    // remove all trailing slashes
    here = path1.end();
    here--;
    begin = path1.begin();
    while (here != begin && (*here == '/' || *here == '\\')) {
      here--;
    }
    here++;
    path1.erase(here,path1.end());

    // add one final slash
#ifdef __WIN32__
    path1 += "\\";
#else
    path1 += "/";
#endif 
  }

  // remove all slashes from the start of path2
  here = path2.begin();
  end = path2.end();
  while (here != end && (*here == '/' || *here == '\\')) {
    here++;
  }
  path2.erase (path2.begin(), here);
  
  text_t fullpath = path1 + path2;

  // make sure all the right slashes are used
  here = fullpath.begin();
  end = fullpath.end();
  while (here != end) {
#ifdef __WIN32__
    if (*here == '/') *here = '\\';
#else
    if (*here == '\\') *here = '/';
#endif
    here ++;
  }
  return fullpath;
}

text_t filename_cat (text_t path1, text_t path2, text_t path3) {
  return filename_cat(filename_cat(path1,path2),path3);
}

text_t filename_cat (text_t path1, text_t path2, text_t path3, text_t path4) {
  return filename_cat(filename_cat(filename_cat(path1,path2),path3),path4);
}

text_t filename_cat (text_t path1, text_t path2, text_t path3, text_t path4, 
		     text_t path5) {
  return filename_cat(filename_cat(filename_cat(filename_cat(path1,path2),path3),
				   path4),path5);
}

// returns true if filename can be opened
bool file_exists (const text_t &filename) {
  char *cstr = filename.getcstr();
#ifdef GSDL_USE_IOS_H
  ifstream filestream (cstr, ios::in | ios::nocreate);
#else
  ifstream filestream (cstr, ios::in);
#endif
  delete cstr;

  if (filestream) {
    // file exists
    filestream.close ();
    return true;
  }

  // file does not exist
  return false;
}

// returns true if filename can be opened
bool file_writable (const text_t &filename) {
  char *cstr = filename.getcstr();
#ifdef GSDL_USE_IOS_H
  ifstream filestream (cstr, ios::out | ios::nocreate);
#else
  ifstream filestream (cstr, ios::out);
#endif
  delete cstr;

  if (filestream) {
    // file exists
    filestream.close ();
    return true;
  }

  // file does not exist
  return false;
}

#if defined(__WIN32__) && !defined(__GNUC__)

#include <windows.h>

bool directory_exists (const text_t &dirname) {
  
  WIN32_FIND_DATA FileData;  
  HANDLE hSearch; 
  char *dirpath = dirname.getcstr();
  strcat (dirpath, "\\*");
  
  hSearch = FindFirstFile(dirpath, &FileData); 
  delete dirpath;

  if (hSearch == INVALID_HANDLE_VALUE) {
    return false;
  }
  return true;
}

bool read_dir (const text_t &dirname, text_tarray &filelist) {
  
  WIN32_FIND_DATA FileData;  
  HANDLE hSearch; 
  char *dirpath = dirname.getcstr();
  strcat (dirpath, "\\*");
  
  hSearch = FindFirstFile(dirpath, &FileData); 
  delete dirpath;

  if (hSearch == INVALID_HANDLE_VALUE) {
    return false;
  }
  
  text_t filename = FileData.cFileName;
  if (filename != "." && filename != ".." && filename != "CVS")
    filelist.push_back (filename); 
  
  while (FindNextFile(hSearch, &FileData)) {
    filename = FileData.cFileName;
    if (filename == "." || filename == ".." || filename == "CVS")
      continue;
    filelist.push_back (filename); 
  }
  
  FindClose(hSearch);
  
  return true;
}

#else

#include <dirent.h>

bool directory_exists (const text_t &dirname) {

  char *tmp = dirname.getcstr();
  DIR *dirin = opendir (tmp);
  delete tmp;
  
  if (dirin == NULL) return false;
  closedir (dirin);
  return true;
}

bool read_dir (const text_t &dirname, text_tarray &filelist) {

  filelist.erase (filelist.begin(), filelist.end());
  
  char *tmp = dirname.getcstr();
  DIR *dirin = opendir (tmp);
  delete tmp;
  
  if (dirin == NULL) return false;
  
  dirent *dirp; 

  text_t filename;
  while ((dirp = readdir (dirin)) != NULL) {
    filename = dirp->d_name;
    if (filename == "." || filename == ".." || filename == "CVS")
      continue;
    filelist.push_back (filename);
  }
  closedir (dirin);
  return true;
}

#endif

// I'm sure there's a better way to do this - for now I don't have 
// time to find it though. -- This will probably break compiling 
// under windows ports of gcc
void file_copy (const text_t &fromfile, const text_t &tofile) {

#ifdef __WIN32__
char *fromfilec = fromfile.getcstr();
char *tofilec = tofile.getcstr();
CopyFile (fromfilec, tofilec, FALSE);
delete fromfilec;
delete tofilec;

#else
text_t cp_cmd = "cp " + fromfile + " " + tofile;
char *cp_cmdc = cp_cmd.getcstr();
system (cp_cmdc);
delete cp_cmd_c

#endif
}
