###########################################################################
#
# ConvertToBasPlug.pm -- inherited plugin that supports conversion of source
#                        docs into a format more easily handled, such as
#                        HTML or Text
#
# A component of the Greenstone digital library software
# from the New Zealand Digital Library Project at the 
# University of Waikato, New Zealand.
#
# Copyright (C) 1999 New Zealand Digital Library Project
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################

# Inherited plugin that supports conversion of source docs into a format
# more easily handled, such as HTML or Text.  It does this by overriding
# the read_file function, so the Plugin can first convert the source document
# to HTML or TEXT.  The converted file is then read using BasPlug::read_file
# The read function is overridden in the same way so the converted files
# can be processed before they are deleted.

# tmp_area_convert_file and cleanup_tmp_area are support function.
# They are private -- don't call from outside object!
 

package ConvertToBasPlug;

use BasPlug;

sub BEGIN {
    @ISA = ('BasPlug');
}

use strict;

sub new {
    my $class = shift (@_);
    my $self = new BasPlug ($class, @_);

    $self->{'convert_to_ext'} = undef;
    
    return bless $self, $class;
}


# Run conversion utility on the input file.  Output files generated in
# collection specific 'tmp' directory.  
# The collection tmp area is used to convert Word, PDF etc documents into 
# another format (such as text or HTML) suitable for reading and indexing

sub tmp_area_convert_file {
    my $self = shift (@_);
    my ($output_ext,$input_filename, $textref) = @_;

    # softlink to collection tmp dir
    my $colname = &util::use_collection();
    my $tmp_dirname 
	= &util::filename_cat($ENV{'GSDLHOME'},"collect",$colname,"tmp");
    &util::mk_dir($tmp_dirname) if (!-e $tmp_dirname);

    # derive tmp filename from input filename
    my ($tailname,$dirname,$suffix)
	= File::Basename::fileparse($input_filename,'\..+');
    my $tmp_filename = &util::filename_cat($tmp_dirname,"$tailname$suffix");

    # Remove any white space from filename -- no risk of name collision, and
    # makes latter conversion by utils simpler
    $tmp_filename =~ s/\s+//g; 

    &util::soft_link($input_filename,$tmp_filename);

    my $output_filename 
	= &util::filename_cat($tmp_dirname,"$tailname.$output_ext");
    $output_filename =~ s/\s+//g; 

    my $verbosity = $self->{'verbosity'};
    if ($verbosity>0)
    {
	print STDERR "Converting $tailname$suffix to $self->{'convert_to'}\n";
    }
    
    # run either gs2txt.pl or gs2html.pl to peform the requested conversion
    if (system("gs2$output_ext.pl -verbose $verbosity \"$tmp_filename\"")>0)
    {
	print STDERR "Warning: unable to convert to $output_ext format\n";
    }

    # remove symbolic link to original file
    &util::rm($tmp_filename);

    return $output_filename;
}


# Remove collection specific tmp directory and all its contents.

sub cleanup_tmp_area {
    my $self = shift (@_);

    my $colname = &util::use_collection();
    my $tmp_dirname 
	= &util::filename_cat($ENV{'GSDLHOME'},"collect",$colname,"tmp");
    &util::rm_r($tmp_dirname);
    &util::mk_dir($tmp_dirname);
}



1;

